#!/usr/bin/env python
# -*- coding: utf-8 -*-

from functools import wraps
import bottle
import lib.plugin


def route(path=None, method='GET', func=None, name=None, apply=None, skip=None, **config):
    def decorator(callback):
        rpath = path
        plugins = []
        if isinstance(apply, str):
            cls = ''.join([apply.lower().capitalize(), 'Plugin'])
            cls = getattr(lib.plugin, cls)
            plugins.append(cls())
        elif isinstance(apply, list):
            for plugin in apply:
                cls = ''.join([apply.lower().capitalize(), 'Plugin'])
                cls = getattr(lib.plugin, cls)
                plugins.append(cls())

        if rpath != '/':
            rpath = rpath.rstrip('/')

        callback = bottle.route(path=rpath, method=method, callback=func, name=name,
                                apply=plugins, skip=skip, **config)(callback)
        @wraps(callback)
        def wrapper(*args, **kwargs):
            return callback(*args, **kwargs)
        return wrapper
    return decorator

def _wrap(path, method, **kwargs):
    def decorator(callback):
        callback = route(path=path, method=method, **kwargs)(callback)
        @wraps(callback)
        def wrapper(*args, **kwargs):
            return callback(*args, **kwargs)
        return wrapper
    return decorator

def get(path=None, **kwargs):
    return _wrap(path=path, method='GET', **kwargs)

def post(path=None, **kwargs):
    return _wrap(path=path, method='POST', **kwargs)

def put(path=None, **kwargs):
    return _wrap(path=path, method='PUT', **kwargs)

def delete(path=None, **kwargs):
    return _wrap(path=path, method='DELETE', **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Singleton(object):
    __instance = {}

    def __new__(cls, *args, **kwargs):
        if cls.__name__ not in cls.__instance:
            cls.__instance[cls.__name__] = \
                super(cls.__class__, cls).__new__(cls, *args, **kwargs)
            setattr(cls.__instance[cls.__name__], '__init', True)
        else:
            setattr(cls.__instance[cls.__name__], '__init', False)
        return cls.__instance[cls.__name__]

    def __init__(self, *args, **kwargs):
        if getattr(self, '__init', False):
            # do initialization here
            pass

    @classmethod
    def get_instance(cls):
        try:
            return cls.__instance[cls.__name__]
        except KeyError:
            return cls()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import pwd
import grp
from gevent import monkey
monkey.patch_all()
from gevent import pywsgi
import bottle
from datetime import date
from vendor.daemon import Daemon
from lib.middleware import Dispatcher
from lib.middleware import StripPath
from lib.config import Config
import lib.plugin
from lib.logger import Log
import argparse


class Decanter(Daemon):
    def __init__(self, app, hostname='localhost', port=9000, pidfile = '/var/run/decanter.pid'):
        self.app = app
        self.hostname = hostname
        self.port = int(port)
        self.pidfile = pidfile
        self.config = Config.get_instance()

        # remove all default bottle plugins
        bottle.uninstall(True)
        bottle.DEBUG = self.config.debug
        # install plugins
        self.install(plugins=self.config.plugins)
        if self.config.debug:
            stdout = os.popen('tty').read().strip()
            stderr = os.popen('tty').read().strip()

        super(Decanter, self).__init__(pidfile, stdout=stdout, stderr=stderr)

    def install(self, plugins=[]):
        for plugin in plugins:
            name = ''.join([plugin.lower().capitalize(), 'Plugin'])
            cls = getattr(lib.plugin, name)
            bottle.install(cls())

    def daemonize(self):
        haspid = os.path.isfile(self.pidfile)
        super(Decanter, self).daemonize()

        username = pwd.getpwuid(os.getuid()).pw_name
        if username != self.config.user:
            uid = pwd.getpwnam(self.config.user).pw_uid
            gid = grp.getgrnam(self.config.group).gr_gid
            os.chown(self.pidfile, uid, gid)
            os.setgid(gid)
            os.setuid(uid)

        if not haspid and os.path.isfile(self.pidfile) and not self.config.test:
            print("Starting daemon with pidfile: {0}".format(self.pidfile))

    def run(self):
        try:
            # Such a message is noise during test.
            # 127.0.0.1 - - [yyyy-MM-dd HH:mm:ss] ...
            log = None if self.config.test else 'default'
            server = pywsgi.WSGIServer((self.hostname, self.port), self.app, log=log)
            server.serve_forever()
        except Exception as e:
            print("Could not start server: {0}".format(e))

    def status(self):
        try:
            with open(self.pidfile, 'r') as fp:
                pid = int(fp.read())
            os.kill(pid, 0)
            print("Decanter is running, pidfile: {0}, process: {0}".format(self.pidfile, pid))
        except (OSError, IOError):
            print("Decanter is not running")


def parse_args(source=sys.argv):
    """
    This function will parse command line arguments. To display help and exit
    if the argument is invalid. Will return command, hostname, port and config.
    """
    defaults = {
        'myself': source.pop(0),
        'hostname': 'localhost',
        'port': 9000
    }
    if len(source) == 0:
        source.append(defaults['myself'])

    parser = argparse.ArgumentParser(description='Example: {myself} -h {hostname} -p {port} -c config/devel.py start'.format(**defaults), conflict_handler='resolve')
    parser.add_argument('command', choices=['start', 'stop', 'restart', 'status'])
    parser.add_argument('-h', '--hostname', default=defaults['hostname'])
    parser.add_argument('-p', '--port', type=int, default=defaults['port'])
    parser.add_argument('-c', '--config', required=True, type=argparse.FileType(), help='config must match the location of a module containing decanter required configuration items, i.e. config/devel.py')
    args = parser.parse_args(source)

    # 'type=argparse.FileType()' will confirm the existence of a file. but it open file.
    args.config.close()
    args.config = os.path.relpath(os.path.realpath(args.config.name),
                                  os.path.dirname(os.path.realpath(__file__)))

    return args

if __name__ == '__main__':
    args = parse_args()

    config = Config(args.config)
    app = Dispatcher(StripPath(bottle.default_app()), config)
    pidfile = config.pidfile.format(args.port)

    # make directory to put pid file
    piddir = os.path.dirname(pidfile)
    if not os.path.isdir(piddir):
        os.makedirs(piddir)

    logfile = config.logger['filepath'].format(args.port, date.today())
    # initialize logger
    log = Log(logfile)
    decanter = Decanter(app, args.hostname, args.port, pidfile)

    # execute command
    {
        'start': lambda: decanter.start(),
        'stop': lambda: decanter.stop(),
        'status': lambda: decanter.status(),
        'restart': lambda: decanter.restart()
    }[args.command]()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import gettext
from functools import wraps
from jinja2 import Environment
from jinja2 import FileSystemLoader
from jinja2 import TemplateNotFound
from bottle import JSONPlugin as JsonPlugin
from lib.config import Config




class Jinja2Plugin(object):
    __state = {}
    name = 'jinja2'
    api = 2

    def __init__(self):
        """ Borg Pattern """
        self.__dict__ = self.__state
        if 'env' not in self.__dict__:
            self.config = Config.get_instance()
            basepath = os.path.join(self.config.apppath, 'bundles')
            bundels = [name for name in os.listdir(basepath) if os.path.isdir(os.path.join(basepath, name))]
            views = []

            for bundle in bundels:
                if bundle != 'views':
                    views.append(os.path.join(basepath, bundle, 'views'))

            # add views directory in bundle
            if 'views' in bundels:
                views.append(os.path.join(basepath, 'views'))

            self.env = Environment(loader=FileSystemLoader(views, encoding='utf-8'), extensions=['jinja2.ext.i18n'])
            self.env.install_gettext_translations(gettext.NullTranslations())


    def apply(self, callback, route):
        @wraps(callback)
        def wrapper(*args, **kwargs):
            if self.name in route.skiplist:
                return callback(*args, **kwargs)

            data = callback(*args, **kwargs)
            if not isinstance(data, dict):
                return data

            bundle = callback.__module__.split('.')[-3]
            controller = callback.__module__.split('.')[-1]
            action = callback.__name__
            template = os.path.join(bundle, controller, '.'.join([action, 'html']))
            try:
                tpl = self.env.get_template(template)
            except TemplateNotFound:
                template = os.path.join(controller, '.'.join([action, 'html']))
                tpl = self.env.get_template(template)
            return tpl.render(data)

        return wrapper


    def setup(self, app):
        pass


    def close(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys


class Dispatcher(object):
    """
    dispatch a request to one of the bundles controllers
    """
    def __init__(self, wsgi, config):
        self.app = wsgi
        self.config = config
        self.bundle = None
        self.controller = None

    def route(self, path):
        segs = []
        if path != '/':
            segs = path.strip('/').split('/')
        if not len(segs):
            self.bundle = self.config.default['bundle']
            self.controller = self.config.default['controller']
        else:
            if self.is_bundle(segs[0]):
                self.bundle = segs[0]
                segs.pop(0)
                if not len(segs):
                    self.controller = self.config.default['controller']
                else:
                    if self.is_controller(self.bundle, segs[0]):
                        self.controller = segs[0]
                        segs.pop(0)
                    else:
                        self.controller = self.config.default['controller']
            elif self.is_bundle(self.config.default['bundle']):
                self.bundle = self.config.default['bundle']
                if self.is_controller(self.bundle, segs[0]):
                    self.controller = segs[0]
                    segs.pop(0)
                else:
                    self.controller = self.config.default['controller']

    def dispatch(self):
        try:
            paths = self.config.apppath.strip(os.path.sep).split(os.path.sep)
            app = paths.pop()
            name = '.'.join([app, 'bundles', self.bundle, 'controllers', self.controller])
            if name not in sys.modules:
                appdir = os.sep.join(path for path in paths)
                if appdir not in sys.path:
                    sys.path.insert(0, appdir)
                __import__(name, fromlist=[self.controller])

        except Exception as e:
            print("Dispatcher: {0}".format(e))


    def is_bundle(self, bundle):
        path = os.path.join(self.config.apppath, 'bundles', bundle)
        return os.path.isdir(path)

    def is_controller(self, bundle, controller):
        path = os.path.join(self.config.apppath, 'bundles', bundle, 'controllers', '.'.join([controller, 'py']))
        return os.path.isfile(path)


    def __call__(self, environ, start_response):
        self.route(environ['PATH_INFO'])
        self.dispatch()
        return self.app(environ, start_response)


class StripPath(object):
    def __init__(self, wsgi):
        self.app = wsgi

    def __call__(self, environ, start_response):
        if environ['PATH_INFO'] != '/':
            environ['PATH_INFO'] = environ['PATH_INFO'].rstrip('/')
        return self.app(environ, start_response)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from lib.decorator import route
from lib.decorator import get
from app.common import errors


@get('/', apply='json', skip='jinja2')
def index():
    return {'word': 'Hello Decanter!'}

@get('/home/')
@get('/home/<name>')
def home(name='Decanter'):
    return {'greeting': "Hello {0}!".format(name.lower().capitalize())}



from setuptools import setup, find_packages
import os
setup(
    name = "decanter",
    version = "0.1.0",
    packages = find_packages(),
    scripts = ['decanter/decanter.py'],

    # Required repositories 
    install_requires = [
        'Jinja2==2.6',
        'MarkupSafe==0.15',
        'argparse==1.2.1',
        'bottle==0.11.6',
        'gevent==0.13.8',
        'greenlet==0.4.0',
        'requests==1.2.0',
        'wsgiref==0.1.2',
    ],

    package_data = {
        # If any package contains *.txt, *.rst or *.md files, include them:
        '': ['*.txt', '*.rst', '*.md'],
    },

    # metadata for upload to PyPI
    author = "Andrea Belvedere",
    author_email = "andrea.belvedere@gengo.com",
    description = "A humble web framework based on bottle",
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    license = "BSD",
    keywords = "web framework bottle gengo",
    url = "https://github.com/gengo/decanter",   # project home page, if any

    # could also include long_description, download_url, classifiers, etc.
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet'
    ]

)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import pwd
import grp

debug = True
test = False

# user name of the user who run decanter
user = pwd.getpwuid(os.getuid()).pw_name

# group name of the user who run decanter
group = grp.getgrgid(os.getegid()).gr_name

# if you run '/path/to/decanter.py', it becomes like '/path/to'
decanterpath = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

# file generated by decanter will be placed here
genpath = os.path.join(decanterpath, 'gen')

# pid file
pidfile = os.path.join(genpath, 'run', 'decanter_{0}.py')

# logging
logger = {
    # log directory path, first {0} is the port number and second {1] is the date
    'filepath': os.path.join(genpath, 'log', 'decanter_{0}-{1}.log'),
    # DEBUG, INFO, WARNING, ERROR, FATAL
    'level': 'DEBUG'
}

# the application directory
apppath = os.path.join(decanterpath, 'app')

# default routes
default = {
    'bundle': 'home',
    'controller': 'index'
}

# list of plugins names to install by default
plugins = ['jinja2']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from singleton import Singleton
import re
import sys


class Config(Singleton):
    def __init__(self, config=None):
        """
        module (str): name of a python module or path to the file containing
        configuration parameters
        """
        if getattr(self, '__init', False):
            # do initialization here
            self.__data = self._load(config)

    def _load(self, config):
        config = re.sub(r'^(\.\/)+', '', config)
        parent_count = 0
        while config.startswith('../'):
            config = config[3:]
            parent_count += 1
        if config.endswith('.py'):
            config = '.'.join(config.rsplit('.py')[0].strip(os.path.sep).
                              split(os.path.sep))

        if parent_count != 0:
            base = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
            for i in range(0, parent_count):
                base = os.path.dirname(base)
            sys.path.insert(0, base)

        segs = config.rsplit('.')
        name = segs.pop() if len(segs) > 1 else None
        if name:
            module = __import__(config, fromlist=[name])
        else:
            module = __import__(config)

        if parent_count != 0:
            sys.path.pop(0)

        return module

    def __getattr__(self, name):
        return getattr(self.__data, name, None)

    def __getitem__(self, name):
        return getattr(self.__data, name, None)

    def __contains__(self, name):
        return hasattr(self.__data, name)
